<?php
// $Id$

/**
 * @file
 * Checklist handler functions.
 */

/**
 * Callback for 'node' item type to give options.
 */
function checklist_checklist_item_node_admin_form_form(&$form, &$form_state, $cl, $item = NULL) {
  $form['node-type'] = array(
    '#type' => 'select',
    '#title' => t('Node type to create'),
    '#options' => node_get_types('names'),
    '#default_value' => $item && $item->data && isset($item->data['node-type']) ? $item->data['node-type'] : NULL,
    '#description' => t('Make sure that whichever node type you select, the user has permission to create the type.'),
    '#required' => TRUE,
  );
  return $form;
}

/**
 * Callback for 'node' item type to process the admin form.
 */
function checklist_checklist_item_node_admin_form_submit(&$form, &$form_state, $cl, $item = NULL) {
  return array('node-type' => $form_state['values']['node-type']);
}

/**
 * Callback for 'node' item type to provide allowed_paths.
 */
function checklist_checklist_item_node_allowed_paths($item, $account) {
  return array('node/add/'. $item->data['node-type']);
}

/**
 * Callback for 'node' item type to provide to modify how the item is displayed.
 */
function checklist_checklist_item_node_display_options($item, $account) {
  return array('path' => 'node/add/'. $item->data['node-type']);
}

/**
 * Callback for 'node' item type to process it.
 */
function checklist_checklist_item_node_process_item($item, $account = NULL) {
  static $items = array();
  // If item is the node type, return the item.
  if (!is_object($item)) {
    return isset($items[$item]) ? $items[$item] : NULL;
  }
  // Store the item so hook_nodeapi() can call this function later.
  $items[$item->data['node-type']] = $item;
  return FALSE;
}



/**
 * Callback for 'user' item type to provide allowed_paths.
 */
function checklist_checklist_item_user_allowed_paths($item, $account) {
  return array('checklist/check/'. $item->cliid);
}

/**
 * Callback for 'user' item type to provide values to modify how the item is displayed.
 */
function checklist_checklist_item_user_display_options($item, $account) {
  // Don't display anything if the item can't be unchecked.
  if (empty($item->checked_on) || $item->allow_uncheck) {
    $i = array(
      'path' => 'checklist/check/' . $item->cliid,
    );
    checklist_item_add_js($account);
    // Only add the javascript if this item is supposed to be linked.
    if ($item->link) {
      $i += array(
        'attributes' => array('onclick' => 'return checklist_check(' . $item->cliid . ', "' . url('checklist/check', array('absolute' => TRUE)) . '/");', 'title' => $item->checked_on ? t('Click to uncheck') : t('Click to check')),
        'div_class' => 'clUser',
      );
      checklist_item_add_js($account);
    }
    return $i;
  }
}

/**
 * Callback for 'user' item type to process it.
 */
function checklist_checklist_item_user_process_item($item, $account) {
  $result = 1;
  // User already checked the item.
  if ($item->checked_on) {
    if ($item->allow_uncheck) {
      db_query('DELETE FROM {checklist_user_checks} WHERE uid = %d AND cliid = %d', $account->uid, $item->cliid);
      $result = 0;
    }
  }
  $result &= !$item->checked_on;
  if (isset($_REQUEST['ajax'])) {
    // Call this since this is an ajax call.
    if ($result) {
      _checklist_complete_user_item($item, $account, TRUE, FALSE);
    }
    print(drupal_to_js(array('cliid' => $item->cliid, 'checked' => $result, 'uncheck' => $item->allow_uncheck)));
    exit();
  }
  // This will complete the checklist item (if TRUE) if it makes it here.
  return $result;
}




/**
 * Callback for 'user' item type to provide values to modify how the item is displayed.
 */
function checklist_checklist_item_admin_display_options($item, $account) {
  // Don't display anything if the item can't be unchecked.
  if (user_access('manage user checklist') && (empty($item->checked_on) || $item->allow_uncheck)) {
    checklist_item_add_js($account, 'admin');
    return array(
      'path' => 'checklist/admin/check/'. $account->uid .'/'. $item->cliid,
      'attributes' => array('onclick' => 'return checklist_check('. $item->cliid .', "'. url('checklist/admin/check/'. $account->uid, array('absolute' => TRUE)) .'/");', 'title' => $item->checked_on ? t('Click to uncheck') : t('Click to check')),
      'div_class' => 'clAdmin',
    );
  }
}

/**
 * Helper function to add the javascript needed to a checklist page.
 */
function checklist_item_add_js($account, $type = 'user') {
  static $js_types_added = array();
  if (empty($js_types_added)) {
    drupal_add_js('function checklist_check(cliid, url) {
    $("#checklistItem"+cliid).addClass(\'clWaiting\');
    $.ajax({url: url+cliid, success: checklist_receive, data: {ajax: 1}, dataType: "json"});
    return false;
  }
  function checklist_receive(data) {
    if (data["cliid"]) {
      var box = $("#checklistItem"+data["cliid"]);
      if (data["checked"]) {
        box.addClass("clChecked");
        if (data["uncheck"] == 1) {
          box.siblings("a").attr("title", "'. t('Click to uncheck') .'");
        }
        else {
          box.siblings("a").attr("title", "");
        }
      }
      else {
        box.removeClass("clChecked");
        box.siblings("a").attr("title", "'. t('Click to check') .'");
      }
      box.removeClass("clWaiting");
    }
  }', 'inline');
  }
  if (!isset($js_types_added[$type])) {
    $js_types_added[$type] = TRUE;
    if ($type == 'admin') {
      drupal_add_js('Drupal.behaviors.ajaxChecklistAdminCheck = function(context) {
      $(".clAdmin").click(function() {
        var cliid = $(this).attr("id").substr(13);
        return checklist_check(cliid, "'. url('checklist/admin/check/'. $account->uid, array('absolute' => TRUE)) .'/");
      });
    }', 'inline');
    }
    elseif ($type == 'user') {
      drupal_add_js('Drupal.behaviors.ajaxChecklistUserCheck = function(context) {
      $(".clUser").click(function() {
        var cliid = $(this).attr("id").substr(13);
        return checklist_check(cliid, "'. url('checklist/check', array('absolute' => TRUE)) .'/");
      });
    }', 'inline');
    }
  }
}




/**
 * Callback for 'page_view' item type to give options.
 */
function checklist_checklist_item_page_view_admin_form_form(&$form, &$form_state, $cl, $item = NULL) {
  $form['page-path'] = array(
    '#type' => 'textfield',
    '#title' => t('Page to visit'),
    '#default_value' => $item && $item->data && isset($item->data['page-path']) ? $item->data['page-path'] : '',
    '#description' => t('Enter the url of the internal drupal page the user must visit.'),
    '#required' => TRUE,
  );
  return $form;
}

/**
 * Callback for 'page_view' item type to give options.
 */
function checklist_checklist_item_page_view_admin_form_validate(&$form, &$form_state, $cl, $item = NULL) {
  $path = trim($form_state['values']['page-path']);

  // Use the source path becuase an alias can change without notice.
  $normal_path = drupal_lookup_path('source', $path);
  if ($normal_path) {
    $path = $normal_path;
    drupal_set_message(t('The path entered is an alias. All paths are stored as the base path.'));
  }

  // Make sure the menu item exists.
  if (!($item = menu_get_item($path))) {
    form_set_error('page-path', t('The path must be a recognized internal drupal path.'));
  }
  else {
    $form_state['values']['page-path'] = $path;
  }
}

/**
 * Callback for 'page_view' item type to process the admin form.
 */
function checklist_checklist_item_page_view_admin_form_submit(&$form, &$form_state, $cl, $item = NULL) {
  return array('page-path' => $form_state['values']['page-path']);
}

/**
 * Callback for 'page_view' item type to provide allowed_paths.
 */
function checklist_checklist_item_page_view_allowed_paths($item, $account) {
  return array($item->data['page-path']);
}

/**
 * Callback for 'page_view' item type to provide to modify how the item is displayed.
 */
function checklist_checklist_item_page_view_display_options($item, $account) {
  return array('path' => $item->data['page-path']);
}

/**
 * Callback for 'page_view' item type to process it.
 */
function checklist_checklist_item_page_view_process_item($item, $account = NULL) {
  // By visiting the page, the item is checked off.
  return TRUE;
}




/**
 * Callback for 'node_agree' item type to give options.
 */
function checklist_checklist_item_node_agree_admin_form_form(&$form, &$form_state, $cl, $item = NULL) {
  $default = '';
  if ($item && $item->data && isset($item->data['nid']) && ($node = node_load($item->data['nid']))) {
    $default = "$node->title [nid:$node->nid]";
  }
  $form['nid'] = array(
    '#type' => 'textfield',
    '#title' => t('Agreement node'),
    '#default_value' => $default,
    '#description' => t('Begin entering the title of the node that a user must agree to. Note, you must make sure the user\'s to complete this checklist have the correct permissions to access the node.'),
    '#autocomplete_path' => 'autocomplete/checklist/node_agree',
    '#required' => TRUE,
  );
  return $form;
}

/**
 * Callback for 'node_agree' item type to give options.
 */
function checklist_checklist_item_node_agree_admin_form_validate(&$form, &$form_state, $cl, $item = NULL) {
  $entry = trim($form_state['values']['nid']);
  // Add 5 for '[nid:'.
  $pos_open = strrpos($entry, '[') + 5;
  $pos_close = strrpos($entry, ']');
  $nid = drupal_substr($entry, $pos_open, $pos_close-($pos_open));
  if (!is_numeric($nid) || !($node = node_load($nid))) {
    form_set_error('nid', t('Please enter a valid node titel and select the corrseponding entry.'));
  }
  else {
    $form_state['values']['nid'] = $nid;
  }
}

/**
 * Callback for 'node_agree' item type to process the admin form.
 */
function checklist_checklist_item_node_agree_admin_form_submit(&$form, &$form_state, $cl, $item = NULL) {
  return array('nid' => $form_state['values']['nid']);
}

/**
 * Callback for 'node_agree' item type to provide allowed_paths.
 */
function checklist_checklist_item_node_agree_allowed_paths($item, $account) {
  return array('node/'. $item->data['nid']);
}

/**
 * Callback for 'node_agree' item type to provide to modify how the item is displayed.
 */
function checklist_checklist_item_node_agree_display_options($item, $account) {
  return array('path' => 'node/'. $item->data['nid']);
}

/**
 * Callback for 'node_agree' item type to process it.
 */
function checklist_checklist_item_node_agree_process_item($item, $account = NULL) {
  static $items = array();

  // If item is the nid of the node, return the item.
  if (!is_object($item)) {
    return isset($items[$item]) ? $items[$item] : NULL;
  }

  // Store the item so hook_nodeapi() can call this function later.
  $items[$item->data['nid']] = $item;
  return FALSE;
}

/**
 * Autocomplete callback for node_agree item type.
 */
function checklist_item_autocomplete_node($string = '') {
  $options = array();
  if ($string) {
    $sql = "SELECT nid, title FROM {node} WHERE status=1 AND lower(title) LIKE lower('%%%s%%') ORDER BY title ASC";
    $sql = db_rewrite_sql($sql, 'n', 'nid', array($string));

    $results = db_query_range($sql, $string, 0, 10);
    while ($row = db_fetch_object($results)) {
      $title = check_plain($row->title);
      $options[$title .' [nid:'. $row->nid .']'] = $title;
    }
  }
  print drupal_to_js($options);
  exit();
}

/**
 * Form to Agree to a Node
 */
function checklist_node_agree_form(&$form_state, $item) {
  $form = array();
  $form['#checklist_item'] = $item;
  $form['checklist_submit'] = array(
    '#type' => 'submit',
    '#value' => t('I Agree'),
    '#weight' => 10
  );
  return $form;
}

/**
 * Submit function for the checklist_agree_form form
 * Checks the Checklist item box off
 */
function checklist_node_agree_form_submit($form, &$form_state) {
  _checklist_complete_user_item($form['#checklist_item']);
}
